#
# shinGETsu - P2P anonymous BBS
#
# (c) FukTommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: Util.pm,v 1.13 2004/02/13 07:21:11 fuktommy Exp $
#

package Shingetsu::Util;
use Shingetsu::Config;
use Shingetsu::Extern;

#
# configuration
#
$PI   = 3.1415926535;
$lock = $Shingetsu::Config::lock;

sub lock(;$) {
	if ($^O ne "cygwin") {
		open LOCK, "> $lock" or die "$lock: open failed. $!";
		flock LOCK, 002;
	}
}

sub unlock() {
	if ($^O ne "cygwin") {
		close LOCK;
	}
}

#
# add a line to update list
# args: stamp, id, filename
#
sub addUpdate($$$) {
	my($stamp, $id, $filename) = @_;
	my $updatelist = $Shingetsu::Config::updatelist;
	my $saveupdate = $Shingetsu::Config::saveupdate;
	my @update = ();
	if (open IN, $updatelist) {
		@update = <IN>;
		close IN;
	}
	push @update, "$stamp<>$id<>$filename\n";
	local $_;
	my $now = time;
	Shingetsu::Util::lock();
	open OUT, "> $updatelist" or die "$updatelist: failed to open. $!";
	foreach (@update) {
		local @_ = split /<>/, $_;
		if ($now - $_[0] <= $saveupdate * 60) {
			print OUT;
		}
	}
	close OUT;
	Shingetsu::Util::unlock();
}

#
# nomal random (Box-Muller)
#
sub normal() {
	my($r1, $r2) = (rand, rand);
	return sqrt(-2 * log($r1)) * sin(2 * $PI * $r2);
}

#
# time with error
#
sub time() {
	my $now = CORE::time;
	my $error = int($Shingetsu::Config::timeError * normal());
	return $now + $error;
}

#
# localtime wrapper for Cygwin's bug
#
sub localtime(;@) {
	my $time;
	if (@_) {
		$time = $_[0];
	} else {
		$time = CORE::time();
	}
	if ((defined $ENV{TZ}) && ($ENV{TZ} ne "")) {
		return gmtime($time - substr($ENV{TZ}, 3) * 60*60);
	} else {
		return CORE::localtime($time);
	}
}

#
# MD5 check records
#
sub md5check(@) {
	my(@record) = @_;
	my @buf = ();
	local (@_, $_);
	foreach (@record) {
		@_ = split /<>/, $_, 3;
		if (@_ < 3) {
			next;
		} elsif ($_[1] eq Shingetsu::Extern::md5digest($_[2])) {
			push @buf, $_;
		}
	}
	return @buf;

}

#
# parse cache record
#
sub rec($) {
	local($_) = @_;
	chomp;
	@_ = split /<>/;
	my %rec = (stamp=> shift @_, id=> shift @_);
	foreach (@_) {
		my @buf = split /:/, $_, 2;
		next if (@buf < 2);
		$rec{$buf[0]} = $buf[1];
	}
	return %rec;
}

1;
